@@ -105,7 +105,7 @@ typedef struct {
 
 GType       e_gw_item_get_type (void);
 EGwItem    *e_gw_item_new_empty (void);
-EGwItem    *e_gw_item_new_from_soap_parameter (const char *container, SoupSoapParameter *param);
+EGwItem    *e_gw_item_new_from_soap_parameter (const char *email, const char *container, SoupSoapParameter *param);
 
 EGwItemType e_gw_item_get_item_type (EGwItem *item);
 void        e_gw_item_set_item_type (EGwItem *item, EGwItemType new_type);
@@ -131,6 +131,8 @@ const char *e_gw_item_get_place (EGwItem *item);
 void        e_gw_item_set_place (EGwItem *item, const char *new_place);
 gboolean    e_gw_item_get_completed (EGwItem *item);
 void        e_gw_item_set_completed (EGwItem *item, gboolean new_completed);
+gboolean    e_gw_item_get_is_allday_event (EGwItem *item);
+void	    e_gw_item_set_is_allday_event (EGwItem *item, gboolean is_allday);	
 char*       e_gw_item_get_field_value (EGwItem *item, char *field_name);
 void        e_gw_item_set_field_value (EGwItem *item, char *field_name, char* field_value);
 GList*      e_gw_item_get_email_list (EGwItem *item);
@@ -160,6 +162,7 @@ void        e_gw_item_set_classification (EGwItem *item, const char *new_class);
 
 #define E_GW_ITEM_ACCEPT_LEVEL_BUSY          "Busy"
 #define E_GW_ITEM_ACCEPT_LEVEL_OUT_OF_OFFICE "OutOfOffice"
+#define E_GW_ITEM_ACCEPT_LEVEL_FREE	     "Free" 	
 
 const char *e_gw_item_get_accept_level (EGwItem *item);
 void        e_gw_item_set_accept_level (EGwItem *item, const char *new_level);
@@ -185,20 +188,23 @@ void e_gw_item_set_recurrence_dates (EGwItem  *item, GSList *new_recurrence_date
 int e_gw_item_get_trigger (EGwItem *item);
 void e_gw_item_set_trigger (EGwItem *item, int trigger);
 
+typedef enum {
+	E_GW_ITEM_STAT_ACCEPTED,
+	E_GW_ITEM_STAT_DECLINED,
+	E_GW_ITEM_STAT_NONE
+} ItemStatus; 
+
 typedef struct {
 	char *email;
 	char *display_name;
 	enum {
 		E_GW_ITEM_RECIPIENT_TO,
 		E_GW_ITEM_RECIPIENT_CC,
+		E_GW_ITEM_RECIPIENT_BC,
 		E_GW_ITEM_RECIPIENT_NONE
 	} type;
 
-	enum {
-		E_GW_ITEM_STAT_ACCEPTED,
-		E_GW_ITEM_STAT_DECLINED,
-		E_GW_ITEM_STAT_NONE
-	} status;
+	ItemStatus status;
 } EGwItemRecipient;
 
 gboolean    e_gw_item_append_to_soap_message (EGwItem *item, SoupSoapMessage *msg);
